-- Game (Create Date: 2020.03.19)
-- -----
-- Description: AOE II CORE. Main game script, provides World generation, assets and textures loading and etc.
-- Last Edit: 2020.09.29

LoadClass("World")
LoadClass("Camera")
LoadClass("GlobalGame")
LoadClass("Age")
LoadClass("Player")
LoadClass("Cursor")
LoadClass("EntUnit")
LoadClass("EntObject")

LoadMiniClass("GamePanel")

LoadPart.Texture.Game.Cursors()
LoadPart.Sound.Nature.Wind()

GlobalGame = GlobalGame:new()
GlobalGame:SetDefaults()

me = {
	World = World:new(),
	Camera = Camera:new(),
	Age = Age:new(),
	Player = Player:new(),
	Cursor = Cursor:new()
}

otr = {
	GamePanel = GamePanel:new()
}

me.Player:SetDefaults()
me.Cursor:SetDefaults()
me.World:SetDefaults()

otr.GamePanel:SetDefaults()
otr.GamePanel:SetPanel(me.Player)



--------------------------
-- World Generating
me.World:GenerateEmpty()
wrldtype = Random(1, 2)
wrldTile1 = Random(1, 3)
wrldTile2 = Random(1, 3)

me.World:GenerateLevel(wrldtype, wrldTile1, wrldTile2)

--me.World:GenerateLevel(2, 2)
--------------------------

me.Camera:Reset(me.World)
me.Camera:TeleportTo(0, 0)

me.Cursor:SetTex()

Ent = {
	Unit = {},
	Building = {},
	Object = {}
}

Create = {
	Villager = function(newX, newY)
		Ent.Unit[1] = EntityUnit:new()
		Ent.Unit[1]:ReceiveAsset(Asset.Unit[1])
		Ent.Unit[1]:SetCords(newX, newY)
		me.Player.Pop = me.Player.Pop + 1
	end,
	ForageBush = function(newX, newY)
		Ent.Object[1] = EntityObject:new()
		Ent.Object[1]:ReceiveAsset(Asset.Object[1])
		Ent.Object[1]:SetCords(newX, newY)
	end
}

Create.Villager(60, 100)
Create.ForageBush(30, 200)

me.Player.CL[1] = {TYPE = "Unit", ID = 1}

-----------------------------------------------------------------------------------

-- Debug Bool
db = false
function DEBUG()
	--if pge.controls.held(PGE_CTRL_SELECT) then
		Arial12:activate()
		Arial12:print(10, 10, Col.White, "Camera")
		Arial12:print(10, 20, Col.White, "X: "..me.Camera.X)
		Arial12:print(10, 30, Col.White, "Y: "..me.Camera.Y)
		
		Arial12:print(65, 10, Col.White, "Cursor")
		Arial12:print(65, 20, Col.White, "X: "..me.Cursor.MapX)
		Arial12:print(65, 30, Col.White, "Y: "..me.Cursor.MapY)
	
		Arial12:print(120, 10, Col.White, "CL[1]")
		if me.Player.CL[1] then
			Arial12:print(120, 20, Col.White, "TYPE: "..me.Player.CL[1].TYPE)
			Arial12:print(120, 30, Col.White, "ID: "..me.Player.CL[1].ID)
		end
		
		Arial12:print(192, 10, Col.White, "World Type: "..wrldtype)
		Arial12:print(192, 20, Col.White, "Tile1: "..wrldTile1)
		Arial12:print(192, 30, Col.White, "Tile2: "..wrldTile2)
	--end
end

--------

WindTim = 700
WindTimMax = 1200

function PlayWind()
	local rndwindsnd = Random(1, 3)
	if rndwindsnd == 1 then
		snd.Nature.Wind.A:play()
	elseif rndwindsnd == 2 then
		snd.Nature.Wind.B:play()
	elseif rndwindsnd == 3 then
		snd.Nature.Wind.C:play()
	end
end

function ActionWindTim()
	WindTim = WindTim + 1
	if WindTim >= WindTimMax then PlayWind() WindTim = 0 end
end

-----------------------------------------------------------------------------------
-----------------------------------------------------------------------------------
-----------------------------------------------------------------------------------
-----------------------------------------------------------------------------------
-----------------------------------------------------------------------------------

while true do
	pge.controls.update()
	pge.gfx.startdrawing()
	pge.gfx.clearscreen(Col.Black)
	me.Cursor:InitMove()
	me.Camera:InitMove()
	ActionWindTim()
	
	me.World:RenderTile(me.Camera)
	me.World:RenderObject(me.Camera)
	
	Ent.Unit[1]:InitRender(me.Camera)
	Ent.Unit[1]:InitActions()
	Ent.Unit[1]:InitAnimate()
	
	Ent.Object[1]:InitRender(me.Camera)
	
	me.Cursor:InitSelect(me.Player, Ent.Unit[1], me.Camera, "L")
	me.Cursor:SetUnitTarget(me.Player, me.Camera, "R")
	
	if pge.controls.pressed(PGE_CTRL_SQUARE) then
		if db == true then
			db = false
		elseif db == false then
			db = true
		end
	end
	
	if db then
		DEBUG()
	else
		otr.GamePanel:InitRender(me.Player)
	end
	
	if pge.controls.pressed(PGE_CTRL_START) then break end
	
	me.Cursor:InitRender()
	
	InitScreenshot()
	
	pge.gfx.enddrawing()
	pge.gfx.swapbuffers()
end